/* DOMAndXSLT.cpp

This program illustrates basic techniques for transforming
a DOM source document into a DOM result document using
the MSXML transformNodeToObject method and XSLT.

Michael C. Rawlins, 2003, for InformIT

*/

#include <iostream.h>

#import <msxml4.dll>
using namespace MSXML2;

int main(int argc, char* argv[])
{

  // Local variables and initializations
  HRESULT hResult = S_OK;

  cout << endl << "DOM and XSLT Demonstration" << endl << endl;
  
  //  Initialize the COM library
  hResult = CoInitialize(NULL);
  if (FAILED(hResult))
  {
    cerr << "Failed to initialize COM environment" << endl;
    return 0;
  }

  //  Main try block for MSXML DOM operations
  try
  {
    //  MSXML COM smart pointers
    //  Use the Dccument2 class to enable schema validation
    IXMLDOMDocument2Ptr spDocSource;
    IXMLDOMDocument2Ptr spDocResult;
    IXMLDOMDocument2Ptr spDocStylesheet;
    struct IDispatch * pDispatch;

    //  Create the COM DOM Document objects
    hResult = spDocSource.CreateInstance(__uuidof(DOMDocument40));
    if FAILED(hResult)
    {
      cerr << "Failed to create Source Document instance" << endl;
      return 1;
    }

    hResult = spDocResult.CreateInstance(__uuidof(DOMDocument40));
    if FAILED(hResult)
    {
      cerr << "Failed to create Result Document instance" << endl;
      return 1;
    }

    hResult = spDocStylesheet.CreateInstance(__uuidof(DOMDocument40));
    if FAILED(hResult)
    {
      cerr << "Failed to create Stylesheet Document instance" << endl;
      return 1;
    }

    //  Load the source document
    spDocSource->async =  VARIANT_FALSE; 
    hResult = spDocSource->load("HelloWorld.xml");
    if( hResult != VARIANT_TRUE)
    {
      cout << "Error parsing HelloWorld.xml" << endl;
      return 1;
    }

    //  Load the stylesheet document
    spDocStylesheet->async =  VARIANT_FALSE; 
    hResult = spDocStylesheet->load("HelloWorld.xsl");
    if( hResult != VARIANT_TRUE)
    {
      cout << "Error parsing HelloWorld.xsl" << endl;
      return 1;
    }

    //  The transformNodeToObject method requires a COM VARIANT
    //  as the second argument.  We query the result document
	//	to get a pointer to its implementation of the
	//	IDispatch interface.
    spDocResult->QueryInterface(IID_IDispatch, (void **) &pDispatch);
    VARIANT vResultDoc;
    vResultDoc.vt = VT_DISPATCH;
    vResultDoc.pdispVal = pDispatch;

    //  Perform the transformation using the source document's
    //  transformNodeToObject method
    hResult =
        spDocSource->transformNodeToObject(spDocStylesheet, vResultDoc);
    if FAILED(hResult)
    {
      cout << "Error in performing transformation" << endl;
      return 1;
    }

    //  Save the result document
    hResult = spDocResult->save("ResultDocument.xml");
    if FAILED(hResult)
    {
      cerr << "Failed to save result document" << endl;
      return 1;
    }

  } // End of try block
 
  //  Catch COM exceptions
  catch (_com_error &e)
  {
    cerr << "COM Error" << endl;
    cerr << "Message = " << e.ErrorMessage() << endl;
    return 1;
  }

  //  Release COM resources
  CoUninitialize();

  cout << endl << endl << "Successful Completion" << endl;

  return 0;
}
