/* DOMAndXSLT.java

This program illustrates basic techniques for transforming
a DOM source document into a DOM result document using
JAXP and Xalan.

Michael C. Rawlins, 2003, for InformIT

*/

// Standard libraries
import java.io.*;

// XML packages
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.w3c.dom.*;
import org.apache.xml.serialize.*;

public class DOMAndXSLT
{

  public static void main (String argv [])
    {
      // Local variables
      Document docSource;
      Document docResult;
      Document docXSLT;
      DOMSource dsSource;
      DOMSource dsXSLT;
      DOMResult drResult;
      OutputFormat MyOutputFormat;
      XMLSerializer MySerializer;
      FileOutputStream OutputXML;

      System.out.println("DOM and XSLT Demonstration");

      try
      {
        //  Create a document buider factory, the builder, and load
        //  the source document.  The factory must be namespace aware for
        //  XSLT.
        DocumentBuilderFactory dFactory =
          DocumentBuilderFactory.newInstance();
        dFactory.setNamespaceAware(true);
        DocumentBuilder dBuilder = dFactory.newDocumentBuilder();
        dBuilder.setErrorHandler(new SAXErrorHandler());

        //  Load and parse the source and stylesheet documents
        docSource = dBuilder.parse(new File("HelloWorld.xml"));
        docXSLT = dBuilder.parse(new File("HelloWorld.xsl"));

        //  Create the result document
        docResult = dBuilder.newDocument();

        //  Create DOMSource objects for the source document and stylesheet.
        //  Set the base URI for the system ID so that any relative URIs can
        //  be resolved - not needed for this constrained example, but is good
        //  practice.
        dsSource = new DOMSource(docSource);
        dsSource.setSystemId("HelloWorld.xml");
        dsXSLT = new DOMSource(docXSLT);
        dsXSLT.setSystemId("HelloWorld.xsl");

        //  Create a DOMResult object for the result of the
        //  transformation, and set its node to be the result Document
        drResult = new DOMResult(docResult);

        //  Create a transformer factory, then a transformer from the
        //  stylesheet DOMSource
        TransformerFactory tFactory = TransformerFactory.newInstance();
        Transformer transformer = tFactory.newTransformer(dsXSLT);

        //  Perform the transformation and get the DOM result tree
        transformer.transform(dsSource, drResult);

        //  Save Output XML Document by creating an output stream, an
        //  XMLSerializer, assigning the output stream to the serializer, and
        //  setting the serializer to be a DOM serializer
        OutputXML = new FileOutputStream("Result.xml");
        MySerializer = new XMLSerializer();
        MySerializer.setOutputByteStream(OutputXML);
        MySerializer.asDOMSerializer();

        //  Create the Output Format for the Serializer for
        //    XML, UTF-8, and indentation true
        MyOutputFormat = new OutputFormat("XML","UTF-8",true);
        MySerializer.setOutputFormat(MyOutputFormat);

        //  Write out the document, and close the output stream
        MySerializer.serialize(docResult);
        OutputXML.flush();
        OutputXML.close();
      }

      //  General purpose catch block
      catch (Throwable t)
      {
        t.printStackTrace ();
        System.exit(1);
      }

      // Finish up
      System.out.println("Successful Completion");
      System.exit(0);
    }
}
