/*  SAXErrorHandler.java

Error handler to report SAX errors and warnings

From Chapter 2 of
Using XML with Legacy Business Applications
Copyright (c) 2002 Addison-Wesley Professional

Author: Michael C. Rawlins
Rawlins EC Consulting
PMB 29
14 Canyon Creek Village
Richardson, TX 75080-1602
mike@rawlinsecconsulting.com
972-783-9573

This file is part of several utilities developed for the book.

All utilities developed for the book are free software; you can
redistribute and/or modify them under the terms of the GNU
General Public License, Version 2, as published by the Free
Software Foundation.

These utilities are distributed in the hope that they will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Foobar; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston,
MA  02111-1307  USA

The license is also available on the World Wide Web at
http://www.gnu.org/licenses/gpl.html



*/

// Standard libraries
import java.io.*;

// JAXP packages
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

public class SAXErrorHandler implements ErrorHandler
{

  //  The first three methods are required by the SAX
  //  ErrorHandler interface.  They deal with warnings, errors,
  //  and fatalErrors, respectively.

  public void warning(SAXParseException spe)
    throws SAXException
  {
    System.err.println("\nWarning during parsing " +
      GetSAXExceptionInfo(spe));
  }

  public void error(SAXParseException spe)
    throws SAXException
  {
    String sInfo = "\nError during parsing " +
      GetSAXExceptionInfo(spe);
    throw new SAXException(sInfo);
  }

  public void fatalError(SAXParseException spe)
    throws SAXException
  {
    String sInfo = "\nFatal error during parsing " +
      GetSAXExceptionInfo(spe);
    throw new SAXException(sInfo);
  }

  //  This utility method gets SAX specific information about
  //  the exception
  private String GetSAXExceptionInfo(SAXParseException spe)
  {
    String sInfo = "\n" +
      "URL    = " + spe.getSystemId() + "\n" +
      "Entity = " + spe.getPublicId() + "\n" +
      "Line = " + spe.getLineNumber() + "\n" +
      "Column = " + spe.getColumnNumber() + "\n" +
      "Text = " + spe.getMessage() + "\n";
    return sInfo;
  }
}
